#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals
from collections import namedtuple

from json import TVD_JSON_EPISODE

class Episode(namedtuple('Episode', ['series', 'season', 'episode'])):
    """
    Parameters
    ----------
    series : str
    season : int
    episode : int
    """

    def __new__(cls, series, season, episode):
        return super(Episode, cls).__new__(cls, series, season, episode)

    def __str__(self):
        return '%s.Season%02d.Episode%02d' % (
            self.series, self.season, self.episode)

    def for_json(self):
        """Returns {TVD_JSON_EPISODE: (series, season, episode)}"""
        return {
            TVD_JSON_EPISODE: (self.series, self.season, self.episode)
        }

    @classmethod
    def from_json(cls, data):
        """
        {TVD_JSON_EPISODE: (series, season, episode)} --> Episode
        """
        series, season, episode = data[TVD_JSON_EPISODE]
        return cls(series=series, season=season, episode=episode)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

import logging
import wave
import contextlib
import yaml
from pkg_resources import resource_filename

from ..core import Episode
from resource import ResourceMixin
from path import PathMixin
from rip import RipMixin

CONFIG_HUMAN_READABLE_NAME = 'name'
CONFIG_ORIGINAL_LANGUAGE = 'language'
CONFIG_EPISODES = 'episodes'
CONFIG_RESOURCES = 'resources'


class Plugin(ResourceMixin, PathMixin, RipMixin):

    def __init__(self, root):
        super(Plugin, self).__init__()

        self.tvd_dir = root

        # obtain path to YAML configuration file and load it
        path = resource_filename(self.__class__.__name__, 'tvd.yml')
        with open(path, mode='r') as f:
            self.config = yaml.load(f)

        # human readable name is obtained from YAML configuration file
        self.name = self.config[CONFIG_HUMAN_READABLE_NAME]

        # original language is obtained from YAML configuration file
        self.language = self.config[CONFIG_ORIGINAL_LANGUAGE]

        # list of episodes according to configuration file
        # e.g. episodes: [17, 23, 23] ==> season 1 (2, 3) with 17 (23, 23) eps
        episodes = self.config.get(CONFIG_EPISODES, [])
        self.episodes = [
            Episode(
                series=self.__class__.__name__,
                season=season_index + 1,
                episode=episode_index + 1
            )
            for season_index, n_episodes in enumerate(episodes)
            for episode_index in range(n_episodes)
        ]

        # load 'resources' section from YAML configuration file
        resources = self.config.get(CONFIG_RESOURCES, {})
        self.init_resource(resources)

    def get_episode_duration(self, episode):
        """Get episode duration from .wav file

        Parameters
        ----------
        episode : Episode

        """

        wav = self.path_to_audio(episode, language=self.language)
        # TODO -- check if wav file exists

        with contextlib.closing(wave.open(wav, 'r')) as f:
            frames = f.getnframes()
            rate = f.getframerate()

        duration = frames / float(rate)
        return duration

    def __str__(self):
        return self.__class__.__name__



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

from command import CommandWrapper


class SndFileResample(CommandWrapper):
    """Dump DVD to disk.

    Parameters
    ----------
    sndfile_resample : str, optional.
        Absolute path to `sndfile_resample`
        in case it is not reachable from PATH.

    """

    def __init__(self, sndfile_resample=None):

        if sndfile_resample is None:
            sndfile_resample = 'sndfile-resample'

        super(SndFileResample, self).__init__(sndfile_resample)

    def to16kHz(self, original, resampled):
        """
        Parameters
        ----------
        original, resampled : str
            Path to `original` and `resampled` wave file
        """

        options = [
            '-to', '16000',
            '-c', '1',
            original,
            resampled,
        ]

        self.run_command(options=options, env=None)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

import subprocess
import os


class CommandWrapper(object):

    """
    Parameters
    ----------
    command : str
        Command (e.g. ls, mkdir or ffmpeg)
    exists_params : iterable, optional
        Parameters passed to command when checking its existence.
        This parameters was introduced to deal with commands that
        do real things even when no parameters are given (e.g. vobcopy needs
        parameter -h to avoid copying DVD)
    """

    def __init__(self, command, exists_params=None):
        super(CommandWrapper, self).__init__()

        # make sure exists_params is a (potentially empty) list
        if exists_params is None:
            exists_params = []
        exists_params = list(exists_params)

        if self.command_exists(command, exists_params):
            self.command = command

        else:
            message = 'Could not find command "{command}"'
            raise ValueError(message.format(command=command))

    @classmethod
    def command_exists(cls, command, exists_params):
        """Check whether `command` exists

        >>> CommandWrapper.command_exists('ls')
        True
        >>> CommandWrapper.command_exists('sl')
        False
        >>> CommandWrapper.command_exists('/bin/ls')
        True
        """

        try:
            with open(os.devnull, mode='w') as _:
                subprocess.Popen(
                    [command] + exists_params,
                    stdout=_, stderr=_
                ).communicate()
        except OSError as e:
            if e.errno == os.errno.ENOENT:
                return False
        return True

    def run_command(self, options=None, env=None):
        """
        Parameters
        ----------
        options : iterable, optional
        env : dict, optional
        """

        if options is None:
            options = []
        else:
            options = list(options)

        cmd = [self.command] + options

        try:
            with open(os.devnull, mode='w') as _:
                subprocess.check_call(cmd, stderr=_, stdout=_, env=env)

        except Exception, e:
            raise e  # TODO: better handling

    def get_output(self, options=None, env=None):
        """
        Parameters
        ----------
        options : iterable, optional
        env : dict, optional
        """

        if options is None:
            options = []
        else:
            options = list(options)

        cmd = [self.command] + options

        try:
            with open(os.devnull, mode='w') as _:
                stdout = subprocess.check_output(cmd, stderr=_, env=env)

        except Exception, e:
            raise e  # TODO: better handling

        return stdout



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""TVD reproduction script -- tvd.niderb.fr

* /list/ mode returns the list of installed TVD plugins.
* /dump/ mode dumps TV series DVDs onto disk.
* /rip/ mode extracts videos, audio tracks and subtitles from dumped DVDs.
* /stream/ mode reencodes videos for streaming.
* /metadata/ mode downloads available metadata.

Usage:
    create list
    create dump [options] [--dvd=<mount>] [--vobcopy=<p>] <tvd> <series> <season> <disc>
    create rip [options] [--lsdvd=<p> --HandBrakeCLI=<p> --mencoder=<p> --vobsub2srt=<p> --avconv=<p> --tessdata=<p> --sndfile-resample=<p>] <tvd> <series> <season>
    create stream [options] [--avconv=<p>] <tvd> <series>
    create metadata [options] <tvd> <series>

Arguments:
    <tvd>     Path to TVD root directory.
    <series>  Series plugin (use "create list" to get the list of series plugins).
    <season>  Season number (e.g. 1 for first season).
    <disc>    Disc number (e.g. 1 for first disc).

Options:
    -f --force                Overwrite existing files.
    --verbose                 Be verbose.

    -d <mount> --dvd=<mount>  DVD mount point [default: /dvd].
    --vobcopy=<p>             Path to "vobcopy" (in case it is not in PATH).
    --lsdvd=<p>               Path to "lsdvd" (in case it is not in PATH).
    --HandBrakeCLI=<p>        Path to "HandBrakeCLI" (in case it is not in PATH).
    --mencoder=<p>            Path to "mencoder" (in case it is not in PATH).
    --vobsub2srt=<p>          Path to "vobsub2srt" (in case it is not in PATH).
    --tessdata=<p>            Path to "tessdata" directory prefix [default: /tessdata].
    --avconv=<p>              Path to "avconv" (in case it is not in PATH).
    --sndfile-resample=<p>    Path to "sndfile-resample" (in case it is not in PATH).
"""

from __future__ import unicode_literals

import re
import logging
from path import path

import tvd
import tvd.core.json
from tvd import Episode
from rip import TVSeriesDVDSet
from rip import Vobcopy
from rip import HandBrakeCLI
from rip import MEncoder
from rip import VobSub2SRT
from rip import AVConv
from rip import SndFileResample

# -------------------------------------------------------------------------


def do_dump(
    series, season, disc,
    dvd=None, vobcopy=None, force=False,
    verbose=False
):

    if verbose:
        logging.basicConfig(level=logging.INFO)

    # tools
    vobcopy = Vobcopy(vobcopy=vobcopy)

    # full directory resulting from vobcopy dump
    full_name = path(series.path_to_dump(season, disc))
    logging.info('Dumping to {full_name}'.format(full_name=full_name))

    # make sure it does not exist already
    if full_name.exists() and not force:
        logging.error('{name} already exists'.format(name=full_name))

    else:
        dump_to, name = full_name.splitpath()
        # create 'dump_to' directory if needed
        path(dump_to).makedirs_p()
        # dump
        vobcopy(dump_to, name=name, dvd=dvd)

    # -------------------------------------------------------------------------


def do_rip(
    series, season,
    lsdvd=None, handbrake=None, mencoder=None, vobsub2srt=None,
    tessdata=None, avconv=None, sndfile_resample=None, force=False,
    verbose=False
):

    if verbose:
        logging.basicConfig(level=logging.INFO)

    original_language = series.language

    def _audio_cmp((i1, l1), (i2, l2)):
        if l1 == original_language:
            return -1
        if l2 == original_language:
            return 1
        else:
            return cmp(i1, i2)

    # tools
    handbrake = HandBrakeCLI(handbrake=handbrake)
    mencoder = MEncoder(mencoder=mencoder)
    vobsub2srt = VobSub2SRT(vobsub2srt=vobsub2srt, tessdata=tessdata)
    avconv = AVConv(avconv=avconv)
    sndfile_resample = SndFileResample(sndfile_resample=sndfile_resample)

    # gather list of disc available for requested series/season
    dump_to = path(series.path_to_dump(1, 1)).dirname()
    disc_pattern = 'Season{season:02d}.Disc*'.format(season=season)
    dvds = path(dump_to).listdir(pattern=disc_pattern)
    dvds = [str(d) for d in dvds]

    logging.debug('Found {number:d} DVDs'.format(number=len(dvds)))
    for d, dvd in enumerate(dvds):
        logging.debug('{d} - {dvd}'.format(d=d + 1, dvd=dvd))

    # create TV series DVD set
    seasonDVDSet = TVSeriesDVDSet(
        series, season, dvds, lsdvd=lsdvd)

    for episode, dvd, title in seasonDVDSet.iter_episodes():

        logging.info('Ripping {episode}'.format(episode=episode))

        dump_to = dvd.dvd

        # get audio tracks as [(1, 'en'), (2, 'fr'), ...]
        audio = list(title.iter_audio())

        # move audio tracks in original language at the beginning
        audio.sort(cmp=_audio_cmp)

        # get subtitle tracks as [1, 2, 3, 4, 5, ...]
        subtitles = [index for index, __ in title.iter_subtitles()]

        # rip episode into .mkv
        handbrake_to = series.path_to_video(episode)

        logging.info('mkv: {to}'.format(to=handbrake_to))

        # make sure output file does not exist already
        if path(handbrake_to).exists() and not force:
            logging.error(
                '{name} already exists'.format(name=handbrake_to))

        else:
            # create containing directory if needed
            path(handbrake_to).dirname().makedirs_p()
            # rip title into mkv
            handbrake.extract_title(
                dump_to, title.index, handbrake_to,
                audio=audio, subtitles=subtitles
            )

        # extract subtitles
        for index, language in title.iter_subtitles():

            rip_srt_to = series.path_to_subtitles(episode, language=language)

            logging.info('srt: {to}'.format(to=rip_srt_to))

            # make sure output file does not exist already
            if path(rip_srt_to).exists() and not force:
                logging.error('{s} already exists'.format(s=rip_srt_to))

            else:
                # create containing directory if needed
                path(rip_srt_to).dirname().makedirs_p()
                try:
                    # extract .sub and .idx
                    mencoder_to = str(path(rip_srt_to).splitext()[0])
                    mencoder.vobsub(
                        dump_to, title.index, language, mencoder_to)
                    # ... to srt
                    vobsub2srt(mencoder_to, language)
                except Exception:
                    logging.error('srt: {to} FAILED'.format(to=rip_srt_to))

        # extract audio tracks
        for stream, (index, language) in enumerate(audio):

            rip_wav_to = series.path_to_audio(episode, language=language)

            logging.info('wav: {to}'.format(to=rip_wav_to))

            # make sure output file does not exist already
            if path(rip_wav_to).exists() and not force:
                logging.error('{w} already exists'.format(w=rip_wav_to))

            else:
                # create containing directory if needed
                path(rip_wav_to).dirname().makedirs_p()
                # .wav --> .raw.wav
                avconv_to = path(rip_wav_to).splitext()[0] + '.raw.wav'
                avconv_to = str(avconv_to)
                # extract raw audio
                avconv.audio_track(handbrake_to, stream + 1, avconv_to)
                # sndfile-resample avconv_to --> rip_wav_to
                sndfile_resample.to16kHz(avconv_to, rip_wav_to)

# -------------------------------------------------------------------------


def do_stream(series, avconv=None, force=False, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO)

    avconv = AVConv(avconv=avconv)

    # find all SERIES.Season**.Episode**.mkv files
    # in TVD/SERIES/dvd/rip/
    # and store all corresponding Episode in episodes list
    episodes = []

    dummy = Episode(series=series.__class__.__name__, season=1, episode=1)
    handbrake_to = series.path_to_video(dummy)
    mkvs = path(handbrake_to).dirname().listdir()

    p = re.compile(
        '{series}.Season([0-9][0-9]).Episode([0-9][0-9]).mkv'.format(
            series=series)
    )
    for mkv in mkvs:
        m = p.search(mkv)
        if m is None:
            continue
        season, episode = m.groups()
        episode = Episode(
            series=series, season=int(season), episode=int(episode))
        episodes.append(episode)

    logging.debug('Found {number:d} episodes'.format(number=len(episodes)))
    for e, episode in enumerate(episodes):
        logging.debug('{e:d} - {episode}'.format(e=e + 1, episode=episode))

    # process every episode
    for episode in episodes:

        # path to mkv file
        handbrake_to = series.path_to_video(episode)

        # extract video in series original language
        # in several format (ogg, mp4, webm) for streaming

        stream = 1  # rip mode made sure first audio stream
                    # is in original language

        # -- ogv --
        ogv_to = series.path_to_stream(episode, format='ogv')
        logging.info('ogv: {to}'.format(to=ogv_to))

        # do not re-generate existing file
        if path(ogv_to).exists() and not force:
            logging.error('{to} already exists.'.format(to=ogv_to))

        else:
            # create containing directory if needed
            path(ogv_to).dirname().makedirs_p()
            avconv.ogv(handbrake_to, stream, ogv_to)

        # -- mp4 --
        mp4_to = series.path_to_stream(episode, format='mp4')

        logging.info('mp4: {to}'.format(to=mp4_to))

        # do not re-generate existing file
        if path(mp4_to).exists() and not force:
            logging.error('{to} already exists.'.format(to=mp4_to))

        else:
            # create containing directory if needed
            path(mp4_to).dirname().makedirs_p()
            avconv.mp4(handbrake_to, stream, mp4_to)

        # -- webm --
        webm_to = series.path_to_stream(episode, format='webm')
        logging.info('webm: {to}'.format(to=webm_to))

        # do not re-generate existing file
        if path(webm_to).exists() and not force:
            logging.error('{to} already exists.'.format(to=webm_to))

        else:
            # create containing directory if needed
            path(webm_to).dirname().makedirs_p()
            avconv.webm(handbrake_to, stream, webm_to)

# -------------------------------------------------------------------------


def do_metadata(series, force=False, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO)

    # loop on all available resources
    for episode, resource_type in series.iter_resources(
        resource_type=None, episode=None, data=False
    ):

        # save resource to PATTERN_RESOURCE in JSON format
        json_to = series.path_to_resource(episode, resource_type)
        logging.info('{episode}: downloading "{resource}".'.format(
            episode=episode, resource=resource_type))

        # do not re-generate existing file
        if path(json_to).exists() and not force:
            logging.info('{episode}: "{resource}" already exists.'.format(
                episode=episode, resource=resource_type))
            continue

        try:
            resource = series.get_resource(resource_type, episode)
        except Exception, e:
            logging.error('{episode}: failed to download "{resource}".'.format(
                episode=episode, resource=resource_type))
            logging.error(e)
            continue

        # create containing directory if needed
        path(json_to).dirname().makedirs_p()
        tvd.core.json.dump(resource, json_to)

# -------------------------------------------------------------------------


def do_list():
    for s in sorted(tvd.series_plugins):
        print s

# -------------------------------------------------------------------------

if __name__ == '__main__':

    from docopt import docopt
    ARGUMENTS = docopt(__doc__, version=tvd.__version__)

    # /list/ mode
    if ARGUMENTS['list']:
        do_list()

    # /dump/ mode
    elif ARGUMENTS['dump']:
        # initialize series plugin
        series = tvd.series_plugins[ARGUMENTS['<series>']](ARGUMENTS['<tvd>'])
        do_dump(
            series,
            int(ARGUMENTS['<season>']),
            int(ARGUMENTS['<disc>']),
            dvd=ARGUMENTS['--dvd'],
            vobcopy=ARGUMENTS['--vobcopy'],
            force=ARGUMENTS['--force'],
            verbose=ARGUMENTS['--verbose']
        )

    elif ARGUMENTS['rip']:
        # initialize series plugin
        series = tvd.series_plugins[ARGUMENTS['<series>']](ARGUMENTS['<tvd>'])
        do_rip(
            series,
            int(ARGUMENTS['<season>']),
            lsdvd=ARGUMENTS['--lsdvd'],
            handbrake=ARGUMENTS['--HandBrakeCLI'],
            mencoder=ARGUMENTS['--mencoder'],
            vobsub2srt=ARGUMENTS['--vobsub2srt'],
            tessdata=ARGUMENTS['--tessdata'],
            avconv=ARGUMENTS['--avconv'],
            sndfile_resample=ARGUMENTS['--sndfile-resample'],
            force=ARGUMENTS['--force'],
            verbose=ARGUMENTS['--verbose']
        )

    elif ARGUMENTS['stream']:
        # initialize series plugin
        series = tvd.series_plugins[ARGUMENTS['<series>']](ARGUMENTS['<tvd>'])
        do_stream(
            series,
            avconv=ARGUMENTS['--avconv'],
            force=ARGUMENTS['--force'],
            verbose=ARGUMENTS['--verbose']
        )

    elif ARGUMENTS['metadata']:
        # initialize series plugin
        series = tvd.series_plugins[ARGUMENTS['<series>']](ARGUMENTS['<tvd>'])
        do_metadata(
            series,
            force=ARGUMENTS['--force'],
            verbose=ARGUMENTS['--verbose']
        )



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

from command import CommandWrapper


class MEncoder(CommandWrapper):
    """Rip previously dumped DVD.

    Parameters
    ----------
    mencoder : str, optional.
        Absolute path to `mencoder` in case it is not reachable from PATH.

    """

    def __init__(self, mencoder=None):

        if mencoder is None:
            mencoder = 'mencoder'

        super(MEncoder, self).__init__(mencoder)

    def vobsub(self, vobcopy_to, title, language, to):
        """
        Extract vobsub from DVD title

        Parameters
        ----------
        vobcopy_to : str
            Path to 'vobcopy' output
        title : int
            Title to process
        language : str
            Language code (e.g. "en", "fr", "es", "de")
        to : str
            Path to output (without extension)
        """

        options = [
            'dvd://{title:d}'.format(title=title),
            '-dvd-device', vobcopy_to,  # TVD/TheBigBangTheory/dvd/dump/Season01.Disc01
            '-o', '/dev/null',
            '-nosound',
            '-ovc', 'copy',
            '-vobsubout', to,
            '-slang', language,
        ]

        self.run_command(options=options)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

# Vobcopy 1.2.0 - GPL Copyright (c) 2001 - 2009 robos@muon.de
#
# Usage: vobcopy
# if you want the main feature (title with most chapters)
# you don't need _any_ options!
# Options:
# [-m (mirror the whole dvd)]
# [-M (Main title - i.e. the longest (playing time) title on the dvd)]
# [-i /path/to/the/mounted/dvd/]
# [-n title-number]
# [-t <your name for the dvd>]
# [-o /path/to/output-dir/ (can be "stdout" or "-")]
# [-f (force output)]
# [-V (version)]
# [-v (verbose)]
# [-v -v (create log-file)]
# [-h (this here ;-)]
# [-I (infos about title, chapters and angles on the dvd)]
# [-1 /path/to/second/output/dir/] [-2 /.../third/..] [-3 /../] [-4 /../]
# [-b <skip-size-at-beginning[bkmg]>]
# [-e <skip-size-at-end[bkmg]>]
# [-O <single_file_name1,single_file_name2, ...>]
# [-q (quiet)]
# [-w <watchdog-minutes>]
# [-x (overwrite all)]
# [-F <fast-factor:1..64>]
# [-l (large-file support for files > 2GB)]

from __future__ import unicode_literals

from command import CommandWrapper


class Vobcopy(CommandWrapper):
    """Dump DVD to disk.

    Parameters
    ----------
    vobcopy : str, optional.
        Absolute path to `vobcopy` in case it is not reachable from PATH.

    """

    def __init__(self, vobcopy=None):

        if vobcopy is None:
            vobcopy = 'vobcopy'

        super(Vobcopy, self).__init__(vobcopy, exists_params=['-h'])

    def __call__(self, to, name=None, dvd=None):
        """Dump DVD to disk

        Parameters
        ----------
        to : str
            Path to output directory
        dvd : str, optional
            Path to the mounted DVD

        """

        options = [
            '-q',      # quiet
            '-m',      # mirror the whole dvd
            '-o', to   # path to output directory
        ]

        if name:
            options.extend(['-t', name])  # DVD name

        if dvd:
            options.extend(['-i', dvd])   # /path/to/the/mounted/dvd

        self.run_command(options=options)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

import os
import logging
import subprocess
import numpy as np
from lxml import objectify
from ..core import Episode


class DVD(object):
    """

    Parameters
    ----------
    dvd : str, optional
        Path to dumped DVD directory (with "vobcopy -m -o `dvd`").
        Default uses physical DVD.
    lsdvd : str, optional
        Path to `lsdvd` command line tool (default to "lsdvd")
    xml : str or file, optional
        Use XML file precomputed by `lsdvd -Ox -avs`.
        When provided, both `dvd` and `lsdvd` parameters have no effect.
    """

    def __init__(self, dvd=None, lsdvd=None, xml=None):

        super(DVD, self).__init__()

        # path to `lsdvd` command line tool
        if lsdvd is None:
            lsdvd = 'lsdvd'

        # by default `lsdvd` will look for physical DVD
        self.dvd = '' if dvd is None else dvd

        if xml is None:

            # use lsdvd command line tool to extract DVD structure
            # TODO: error handle
            with open(os.devnull, mode='w') as f:
                xml_content = subprocess.check_output(
                    [lsdvd, '-Ox', '-avs', self.dvd], stderr=f
                )

        elif isinstance(xml, file):

            xml_content = xml.read()

        elif isinstance(xml, str):

            with open(xml, mode='r') as f:
                xml_content = f.read()

        # get rid of potentially buggy characters
        # TODO: find a better fix
        xml_content = xml_content.replace('\xff', '?').replace('&', '')

        # titles sorted in index order
        self.titles = sorted(
            [DVDTitle(t) for t in objectify.fromstring(str(xml_content)).track[:]],
            key=lambda title: title.index,
        )

    def __repr__(self):
        return '<DVD dvd://{path}>'.format(path=self.dvd)


class DVDTitle(object):

    def __init__(self, track):
        super(DVDTitle, self).__init__()

        self.index = int(track.ix)
        self.duration = float(track.length)

        # load audio tracks
        self.audios = {}
        if hasattr(track, 'audio'):
            for a in track.audio[:]:
                audio = DVDAudio(a)
                if audio.langcode in self.audios:
                    msg = (
                        'Title {index} contains multiple audio tracks '
                        'with language {langcode}.'
                    ).format(index=self.index, langcode=audio.langcode)
                    logging.debug(msg)
                    continue

                self.audios[audio.langcode] = audio

        # load subtitles
        self.subtitles = {}
        if hasattr(track, 'subp'):
            for s in track.subp[:]:
                subtitle = DVDSubtitle(s)
                if subtitle.langcode in self.subtitles:
                    msg = (
                        'Title {index} contains multiple subtitles tracks '
                        'with language {langcode}.'
                    ).format(index=self.index, langcode=subtitle.langcode)
                    logging.debug(msg)
                    continue

                self.subtitles[subtitle.langcode] = subtitle

    def __str__(self):
        return "<DVDTitle dvd://%d>" % self.index

    def __repr__(self):
        return "<DVDTitle dvd://%d>" % self.index

    def iter_audio(self):
        return iter(sorted([
            (audio.index, langcode)
            for langcode, audio in self.audios.iteritems()
        ]))

    def iter_subtitles(self):
        return iter(sorted([
            (subtitle.index, langcode)
            for langcode, subtitle in self.subtitles.iteritems()
        ]))


# TODO: make it a namedtuple
class DVDAudio(object):

    def __init__(self, audio):
        super(DVDAudio, self).__init__()
        self.index = int(audio.ix)
        self.language = str(audio.language)
        self.langcode = str(audio.langcode)
        self.frequency = int(audio.frequency)
        self.channel = int(audio.channels)
        self.format = str(audio.format)


# TODO: make it a namedtuple
class DVDSubtitle(object):

    def __init__(self, subp):
        super(DVDSubtitle, self).__init__()
        self.index = int(subp.ix)
        self.language = str(subp.language)
        self.langcode = str(subp.langcode)


class TVSeriesDVD(DVD):
    """

    Parameters
    ----------
    name : str
        Name of the TV series (e.g. "The Big Bang Theory")
    season : int
        Season number (e.g. 1 for first season)
    first : int
        Episode number ()
    dvd : str, optional
        Path to dumped DVD directory (with "vobcopy -m -o `dvd`").
        Default uses physical DVD.
    lsdvd : str, optional
        Path to `lsdvd` command line tool (default to "lsdvd")
    xml : str or file, optional
        Use XML file precomputed by `lsdvd -Ox -avs`.
        When provided, both `dvd` and `lsdvd` parameters have no effect.
    """
    def __init__(self, name, season, first, dvd=None, lsdvd=None, xml=None):
        super(TVSeriesDVD, self).__init__(dvd=dvd, lsdvd=lsdvd, xml=xml)
        self.name = name
        self.season = season
        self.first = first

    def guess_episode_duration(self, number=None):
        """Try and guess the average duration of an episode

        This heuristic relies on the following assumptions:
        - episodes are usually among the longest tracks of the DVD
        - there are at least 3 episodes in every DVD

        Parameters
        ----------
        number : int, optional
            Number of episodes in DVD.

        Returns
        -------
        duration : float
            Average episode duration

        """

        # if number is not known in advance
        # use 3 by default
        if number is None:
            number = 3

        # sort titles by duration (longest first)
        titles, durations = zip(
            *sorted(
                [(t, t.duration) for t in self.titles],
                key=lambda (title, duration): duration,
                reverse=True,
            )
        )

        # estimate episode duration as
        # median of n+1 longest titles
        return np.median(durations[:number+1])

    def find_episode_titles(self, duration=None, number=None):
        """

        This heuristic relies on the following assumptions:
        - episodes are usually among the longest tracks of the DVD
        - there are at least 3 episodes in every DVD
        - the very first track may contain the concatenation of all episodes
        - episodes are stored in chronological order on the DVD

        Parameters
        ----------
        duration : float, optional
            Average episode duration.
            Try to guess it automatically when not provided.
        number : int, optional
            Number of episodes in DVD.
            Try to guess it automatically when not provided.

        Returns
        -------
        episodes : list of `DVDTitle`
            List of titles found to contain each episodes.
        """

        if duration is None:
            target_duration = self.guess_episode_duration(number=number)
        else:
            target_duration = duration

        # duration of first track
        firstDuration = self.titles[0].duration

        # first track is very likely to contain all episodes
        # if its duration is more than two times the episode duration
        if firstDuration > 2 * target_duration:
            firstTitleContainsAll = True
        else:
            firstTitleContainsAll = False

        # detected episodes so far and their total duration
        episode_titles = {}
        sum_duration = 0.

        for title in self.titles:

            # title duration
            duration = title.duration

            # skip title if its duration is not close enough to target duration
            if np.abs(np.log(duration/target_duration)) > np.log(1.2):
                continue

            if number:
                # stop looking for episodes if we found enough already
                if len(episode_titles) > number:
                    break

            # skip track if adding it would not match first title duration
            if firstTitleContainsAll:
                if sum_duration + duration > firstDuration + 0.5 * target_duration:
                    break

            # if we passed all previous test,
            # then it is an episode
            episode = Episode(
                series=self.name,
                season=self.season,
                episode=self.first + len(episode_titles)
            )
            episode_titles[episode] = title
            sum_duration = sum_duration + title.duration

        return episode_titles

    def iter_episodes(self):

        episode_titles = self.find_episode_titles(duration=None, number=None)
        for episode in sorted(episode_titles):
            title = episode_titles[episode]
            yield episode, title


class TVSeriesDVDSet(object):
    """

    Parameters
    ----------
    name : str
        Name of the TV series
    season : int
        Season number of DVD set
    dvds : list
        Disc-ordered paths to dumped DVD directories
        (with "vobcopy -m -o `dvd`")
    lsdvd : str, optional
        Path to `lsdvd` command line tool (default to "lsdvd")
    """

    def __init__(self, name, season, dvds, lsdvd=None):

        super(TVSeriesDVDSet, self).__init__()
        self.name = name
        self.season = season
        self.dvds = []

        first = 1
        for dvd in dvds:
            tvSeriesDVD = TVSeriesDVD(
                name, season, first, dvd=dvd, lsdvd=lsdvd)
            self.dvds.append(tvSeriesDVD)
            first = first + len(tvSeriesDVD.find_episode_titles())

    def iter_episodes(self):

        for dvd in self.dvds:
            for episode, title in dvd.iter_episodes():
                yield episode, dvd, title



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013 Hervé BREDIN (http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from tvd.dvd.tvseries import TVSeriesDVD
import numpy as np
import pandas

from pkg_resources import resource_filename


class test_TVSeriesDVD(object):

    def setup(self):

        # load DVD groundtruth
        self.dvds = pandas.read_csv(
            resource_filename(__package__, 'data/test.csv'),
            converters={
                'series': str,
                'season': int,
                'disc': int,
                'xml': str,
                'episodeDuration': lambda value: float(value)*60,
                'numberOfEpisodes': int,
                'titles': lambda value: [int(t) for t in value.split()]
            }
        )

    def teardown(self):
        pass

    def test_guessDuration(self):

        for _, row in self.dvds.iterrows():

            duration = row['episodeDuration']

            dvd = TVSeriesDVD(
                row['series'], row['season'], row['disc'],
                xml=resource_filename(__package__, 'data/%s' % row['xml'])
            )

            assert np.abs(
                np.log(dvd.guess_episode_duration()/duration)) < np.log(1.1)

    def test_guessDurationWithNumber(self):

        for _, row in self.dvds.iterrows():

            duration = row['episodeDuration']

            dvd = TVSeriesDVD(
                row['series'], row['season'], row['disc'],
                xml=resource_filename(__package__, 'data/%s' % row['xml'])
            )

            assert np.abs(
                np.log(
                    dvd.guess_episode_duration(
                        number=row['numberOfEpisodes'])/duration
                    )
                ) < np.log(1.1)

    def test_findEpisodes(self):

        for _, row in self.dvds.iterrows():

            titles = row['titles']

            dvd = TVSeriesDVD(
                row['series'], row['season'], row['disc'],
                xml=resource_filename(__package__, 'data/%s' % row['xml'])
            )

            episodes = dvd.find_episode_titles()

            assert titles == [e.index for e in episodes]

    def test_findEpisodesWithNumber(self):

        for _, row in self.dvds.iterrows():

            titles = row['titles']

            dvd = TVSeriesDVD(
                row['series'], row['season'], row['disc'],
                xml=resource_filename(__package__, 'data/%s' % row['xml'])
            )

            episodes = dvd.find_episode_titles(number=row['numberOfEpisodes'])

            assert titles == [e.index for e in episodes]

    def test_findEpisodesWithDuration(self):

        for _, row in self.dvds.iterrows():

            titles = row['titles']

            dvd = TVSeriesDVD(
                row['series'], row['season'], row['disc'],
                xml=resource_filename(__package__, 'data/%s' % row['xml'])
            )

            episodes = dvd.find_episode_titles(duration=row['episodeDuration'])

            assert titles == [e.index for e in episodes]

    def test_findEpisodesWithDurationAndNumber(self):

        for _, row in self.dvds.iterrows():

            titles = row['titles']

            dvd = TVSeriesDVD(
                row['series'], row['season'], row['disc'],
                xml=resource_filename(__package__, 'data/%s' % row['xml'])
            )

            episodes = dvd.find_episode_titles(
                number=row['numberOfEpisodes'],
                duration=row['episodeDuration']
            )

            assert titles == [e.index for e in episodes]



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

from command import CommandWrapper


class HandBrakeCLI(CommandWrapper):
    """Rip previously dumped DVD.

    Parameters
    ----------
    handbrake : str, optional.
        Absolute path to `HandBrakeCLI` in case it is not reachable from PATH.

    """

    def __init__(self, handbrake=None):

        if handbrake is None:
            handbrake = 'HandBrakeCLI'

        super(HandBrakeCLI, self).__init__(handbrake)

    def extract_title(
        self, vobcopy_to, title, to,
        quality=18, rate=25,
        audio=None,
        subtitles=None
    ):
        """
        Rip DVD title into .mkv (with selected audio and subtitles tracks)

        Parameters
        ----------
        vobcopy_to : str
            Path to 'vobcopy' output
        title : int
            Title to rip
        to : str
            Path to HandBrake output
        quality : int
            Video quality. Ranges from 51 (low quality) to 0 (high quality).
            Defaults to 18. See https://trac.handbrake.fr/wiki/ConstantQuality
        rate : int
            Fixed frame rate. Defaults to 25.
        audio : list, optional
            Audio tracks with their names
            e.g. [(1, 'en'), (2, 'fr'), (3, 'es')]
        subtitles : list, optional
            Subtitle tracks
            e.g. [1, 2, 3, 4, 5, 6, 7, 8]

        """

        options = [
            '--input', vobcopy_to,
            '--title', str(title),
            '--output', to,
            '--format', 'mkv',
            '--encoder', 'x264',
            '--rate', str(rate),
            '--quality', str(quality),
        ]

        if audio is not None:

            # --audio 1,2,3
            audio_param = ','.join([str(stream) for stream, _ in audio])
            options.extend(['--audio', audio_param])

            # --audio copy,copy,copy
            aencoder_param = ','.join(['copy' for _, _ in audio])
            options.extend(['--aencoder', aencoder_param])

            # --aname en,fr,es
            aname_param = ','.join([name for _, name in audio])
            options.extend(['--aname', aname_param])

        if subtitles is not None:

            # --subtitle
            subtitles_param = ','.join([str(stream) for stream in subtitles])
            options.extend(['--subtitle', subtitles_param])

        self.run_command(options=options)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals
from command import CommandWrapper


class AVConv(CommandWrapper):
    """Convert audio or video streams

    Parameters
    ----------
    avconv : str, optional.
        Absolute path to `avconv` in case it is not reachable from PATH.

    """

    def __init__(self, avconv=None):

        if avconv is None:
            avconv = 'avconv'

        super(AVConv, self).__init__(avconv)

    def audio_track(self, handbrake_to, stream, to):
        """
        Extract audio track from HandBrake output

        Parameters
        ----------
        handbrake_to : str
            Path to input file
        stream : int
        to : str
            Path to output file

        See savvyadmin.com/extract-audio-from-video-files-to-wav-using-ffmpeg

        """

        options = [
            '-y',
            '-i', handbrake_to,
            '-map', '0:{stream:d}'.format(stream=stream),
            '-acodec', 'pcm_s16le',
            '-ac', '1',
            to
        ]

        self.run_command(options=options, env=None)

    # http://www.willus.com/author/streaming2.shtml
    # -map option details can be foundhere :
    # http://libav.org/avconv.html#Advanced-options

    def mp4(self, handbrake_to, audio_stream, to):

        options = [
            '-y',
            '-i', handbrake_to,
            '-map', '0:0,0:0',
            '-i_qfactor', '0.71',
            '-qcomp', '0.6',
            '-qmin', '10',
            '-qmax', '63',
            '-qdiff', '4',
            '-trellis', '0',
            '-vcodec', 'libx264',
            '-b:v', '200k',
            '-map', '0:{stream:d},0:0'.format(stream=audio_stream),
            '-b:a', '56k',
            '-ar', '22050',
            '-ac', '2',
            '-acodec', 'libvo_aacenc',
            to
        ]

        self.run_command(options=options, env=None)

    def webm(self, handbrake_to, audio_stream, to):

        options = [
            '-y',
            '-i', handbrake_to,
            '-map', '0:0,0:0',
            '-qmax', '63',
            '-b:v', '200k',
            '-vcodec', 'libvpx',
            '-map', '0:{stream:d},0:0'.format(stream=audio_stream),
            '-b:a', '56k',
            '-ar', '22050',
            '-ac', '2',
            '-acodec', 'libvorbis',
            to
        ]

        self.run_command(options=options, env=None)

    def ogv(self, handbrake_to, audio_stream, to):

        options = [
            '-y',
            '-i', handbrake_to,
            '-map', '0:0,0:0',
            '-qmax', '63',
            '-b:v', '200k',
            '-vcodec', 'libtheora',
            '-map', '0:{stream:d},0:0'.format(stream=audio_stream),
            '-b:a', '56k',
            '-ar', '22050',
            '-ac', '2',
            '-acodec', 'libvorbis',
            to
        ]

        self.run_command(options=options, env=None)




# This file was generated by 'versioneer.py' (0.8+) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '0.7.4'
version_full = 'bc18a5b07b434e0fb07ae3c959b04f9a47f754e0'
def get_versions(default={}, verbose=False):
    return {'version': version_version, 'full': version_full}




#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals


import logging
import requests
from ..core import Episode
from ..core.json import load as load_json
from pyannote.core import T
import sys


TVD_RESOURCE_URL = 'url'
TVD_RESOURCE_SEASON = 'season'
TVD_RESOURCE_EPISODE = 'episode'
TVD_RESOURCE_SOURCE = 'source'
TVD_RESOURCE_TYPE = 'type'
TVD_ACKNOWLEDGEMENT = """
IN CASE YOU USE '{resource}' RESOURCES, PLEASE CONSIDER CITING:
{reference}
"""


class ResourceMixin(object):

    def init_resource(self, resources):

        # initialize web resources data structure
        # resources[episode] contains all resources for a given episode
        self.resources = {}

        # loop on web resources described in 'resources' section
        for resource_type, resource in resources.iteritems():

            if 'source' in resource:
                sys.stdout.write(TVD_ACKNOWLEDGEMENT.format(
                    resource=resource_type, reference=resource['source']))

            # obtain corresponding 'get_resource' method
            resource_method = self._get_resource_method(resource_type)

            # read resource 'source' from YAML configuration when provided
            source = resource.get(TVD_RESOURCE_SOURCE, None)

            # read resource 'type' from YAML configuration when provided
            data_type = resource.get(TVD_RESOURCE_TYPE, None)

            # loop on all provided URLs
            # NB: some episodes might not have some resources
            for url in resource.get(TVD_RESOURCE_URL, []):

                season_number = url[TVD_RESOURCE_SEASON]
                episode_number = url[TVD_RESOURCE_EPISODE]
                episode = Episode(
                    series=self.__class__.__name__,
                    season=season_number,
                    episode=episode_number
                )

                # add episode to resource main structure
                # in case it is the first time it is encountered
                if episode not in self.resources:
                    self.resources[episode] = {}

                # initialize resource placeholder
                self.resources[episode][resource_type] = {
                    # method to call to get the resource
                    'method': resource_method,
                    # parameters to pass to method to get the resource
                    'params': {
                        'url': url[TVD_RESOURCE_URL],
                        'source': source,
                        'episode': episode
                    },
                    # results of call "method(**params)"
                    # NB: it is set to None until we actually get this resource
                    'result': None,
                    # data type (transcription, annotation, else ?)
                    'type': data_type
                }

    def _get_resource_method(self, resource_type):
        """Get method for given resource

        resource_type : str
            Resource type, as provided in YAML configuration file

        Returns
        -------
        method : callable
            Method defined in inheriting plugin to be used
            to obtain `resource_type` resource

        """
        resource_method = getattr(self, resource_type, None)

        if resource_method is None:

            error = 'Class %s has no method called %s.' % (
                self.__class__.__name__, resource_type)

            raise ValueError(error)

        return resource_method

    def has_resource(self, resource_type, episode):
        """
        Checks whether resource `resource_type` exists
        for given `episode`

        Parameters
        ----------
        resource_type : str
        episode : `tvd.Episode`
        """

        resources = self.resources.get(episode, None)
        if resources is None:
            return False

        resource = resources.get(resource_type, None)
        if resource is None:
            return False

        return True

    def get_resource_from_disk(self, resource_type, episode):
        """Load resource from disk, sotre it in memory and return it

        Parameters
        ----------
        episode : Episode
            Episode
        resource_type : str
            Type of resource

        Returns
        -------
        resource : Timeline, Annotation or Transcription
            Resource of type `resource_type` for requested episode

        Raises
        ------
        Exception
            If the resource is not available on disk

        """

        msg = 'getting {t:s} for {e:s} from disk'
        logging.debug(msg.format(e=episode, t=resource_type))

        path = self.path_to_resource(episode, resource_type)
        result = load_json(path)

        msg = 'saving {t:s} for {e:s} into memory'
        logging.debug(msg.format(e=episode, t=resource_type))

        self.resources[episode][resource_type]['result'] = result

        return result

    def get_resource_from_memory(self, resource_type, episode):
        """Load resource from memory

        Parameters
        ----------
        episode : Episode
            Episode
        resource_type : str
            Type of resource

        Returns
        -------
        resource : Timeline, Annotation or Transcription
            Resource of type `resource_type` for requested episode

        Raises
        ------
        Exception
            If the resource is not available in memory
        """

        msg = 'getting {t:s} for {e:s} from memory'
        logging.debug(msg.format(e=episode, t=resource_type))

        result = self.resources[episode][resource_type]['result']

        if result is None:
            msg = 'resource {t:s} for {e:s} is not available in memory'
            raise ValueError(msg.format(e=episode, t=resource_type))

        return result

    def get_resource_from_plugin(self, resource_type, episode):
        """Load resource from plugin, store it in memory and return it

        Parameters
        ----------
        episode : Episode
            Episode
        resource_type : str
            Type of resource

        Returns
        -------
        resource : Timeline, Annotation or Transcription
            Resource of type `resource_type` for requested episode

        Raises
        ------
        Exception
            If plugin failed to provide the requested resource
        """

        msg = 'getting {t:s} for {e:s} from plugin'
        logging.debug(msg.format(e=episode, t=resource_type))

        resource = self.resources[episode][resource_type]
        method = resource['method']
        params = resource['params']

        T.reset()
        result = method(**params)

        msg = 'saving {t:s} for {e:s} into memory'
        logging.debug(msg.format(e=episode, t=resource_type))

        self.resources[episode][resource_type]['result'] = result

        return result

    def get_resource(self, resource_type, episode, update=False):
        """Get resource

        Parameters
        ----------
        resource_type: str
        episode : Episode
        update : bool, optional

        Returns
        -------
        resource : Timeline, Annotation or Transcription
            Resource of type `resource_type` for requested `episode`

        Raises
        ------
        ValueError
            If plugin failed to provide the requested resource.

        """

        result = None

        if update:
            funcs = [self.get_resource_from_plugin]

        else:
            funcs = [self.get_resource_from_memory,
                     self.get_resource_from_disk,
                     self.get_resource_from_plugin]

        for func in funcs:
            try:
                result = func(resource_type, episode)
                break

            except:
                continue

        if result is None:
            error = 'cannot get {t:s} for episode {e:s}'
            raise ValueError(error.format(t=resource_type, e=episode))

        return result

    def iter_resources(self, resource_type=None, episode=None,
                       data=False, update=True):
        """Resource iterator

        Resources are yielded in episode chronological order
        and resource name alphabetical order

        Parameters
        ----------
        resource_type : str, optional
            When provided, only iterate over this resource type
        episode : `tvd.Episode`, optional
            When provided, only iterate over resources for this episode
        data : boolean, optional
            Whether to yield actual data

        Returns
        -------
        (episode, resource_type[, data]) iterator
        """

        # loop on episodes in airing chronological order
        for _episode in sorted(self.resources):

            # skip this episode if not requested
            if (episode is not None) and \
               (episode != _episode):
                continue

            # loop on resources in name alphabetical order
            for _resource_type in sorted(self.resources[_episode]):

                # skip this resource if not requested
                if (resource_type is not None) and \
                   (resource_type != _resource_type):
                    continue

                # really get this resource
                if data:
                    _data = self.get_resource(_resource_type, _episode,
                                              update=update)
                    yield _episode, _resource_type, _data

                else:
                    yield _episode, _resource_type

    def get_all_resources(self, update=False):

        resources = {}

        for episode, resource_type in self.iter_resources(
            resource_type=None, episode=None, data=False
        ):

            if episode not in resources:
                resources[episode] = {}

            resource = self.get_resource(resource_type, episode, update=update)

            resources[episode][resource_type] = resource

        return resources

    CHARACTER_MAPPING = {
        # hyphen
        ord(u'\u2013'): u"-",    # –
        ord(u'\u2014'): u"-",    # —
        # quote
        ord(u'\u2018'): u"'",    # ‘
        ord(u'\u2019'): u"'",    # ’
        # double-quote
        ord(u'\u201c'): u'"',    # “
        ord(u'\u201d'): u'"',    # ”
        # space
        ord(u'\u200b'): u" ",    #
        ord(u'\xa0'): u" ",      #
        # other
        ord(u'\u2026'): u"...",  # …
        # ord(u'\xe9'): u"é"
    }

    HTML_MAPPING = {
        # hyphen
        u"&#8211;": u"-",
        # quote
        u"&#8216;": u"'",
        u"&#8217;": u"'",
        u"&quot;": '"',
        # double-quote
        u"&#8220;": u'"',
        u"&#8221;": u'"',
        # other
        u"&#8230;": u"..."
    }

    def clean_text(self, udata):

        # apply mapping tables
        udata = udata.translate(self.CHARACTER_MAPPING)
        for old, new in self.HTML_MAPPING.iteritems():
            udata = udata.replace(old, new)
        return udata

    def download_as_utf8(self, url):
        """Download webpage content as UTF-8

        Parameters
        ----------
        url : str
            Webpage URL
        mapping : dict
            Translation table (mapping of Unicode ordinals to Unicode ordinals
            or to Unicode strings).
        Returns
        -------
        text : unicode
            Webpage content as unicode UTF-8 text

        """

        # request URL content with dummy user-agent
        user_agent = {'User-agent': 'TVD'}
        r = requests.get(url, headers=user_agent)

        # get content as UTF-8 unicode
        r.encoding = 'UTF-8'

        return self.clean_text(r.text)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

import sys
from pkg_resources import iter_entry_points

from pyannote.core import T, TStart, TEnd, Transcription
from pyannote.core import Segment, Timeline, Annotation
from core import Episode
from plugin import Plugin

__all__ = [
    'Plugin',
    'Episode',
    'Segment', 'Timeline', 'Annotation',
    'T', 'TStart', 'TEnd', 'Transcription'
]

# plugin_name --> plugin_class
series_plugins = {}

for o in iter_entry_points(group='tvd.series', name=None):

    series = o.name

    plugin = o.load()
    series_plugins[series] = plugin

    setattr(sys.modules[__name__], series, plugin)
    __all__.append(series)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals

import simplejson as json
import pyannote.core.json

TVD_JSON_EPISODE = 'E'


def object_hook(d):
    """
    Usage
    -----
    >>> import simplejson as json
    >>> with open('file.json', 'r') as f:
    ...   json.load(f, object_hook=object_hook)
    """

    from episode import Episode

    if TVD_JSON_EPISODE in d:
        return Episode.from_json(d)

    d = pyannote.core.json.object_hook(d)

    return d

def load(path):
    with open(path, 'r') as f:
        data = json.load(f, encoding='utf-8', object_hook=object_hook)
    return data


def dump(data, path):
    with open(path, 'w') as f:
        json.dump(data, f, encoding='utf-8', for_json=True)



#!/usr/bin/env python
# encoding: utf-8

#
# The MIT License (MIT)
#
# Copyright (c) 2013-2014 CNRS (Hervé BREDIN -- http://herve.niderb.fr/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from __future__ import unicode_literals


class PathMixin(object):

    def path_to_dump(self, season, disc):

        pattern = (
            '{tvd}/{series}/dvd/dump/'
            'Season{season:02d}.Disc{disc:02d}'
        )

        return pattern.format(
            tvd=self.tvd_dir,
            series=self.__class__.__name__,
            season=season,
            disc=disc
        )

    def path_to_video(self, episode):

        pattern = (
            '{tvd}/{series}/dvd/rip/video/'
            '{series}.Season{season:02d}.Episode{episode:02d}.mkv'
        )

        return pattern.format(
            tvd=self.tvd_dir,
            series=episode.series,
            season=episode.season,
            episode=episode.episode,
        )

    def path_to_audio(self, episode, language=None):

        pattern = (
            '{tvd}/{series}/dvd/rip/audio/'
            '{series}.Season{season:02d}.Episode{episode:02d}.{language}.wav'
        )

        if language is None:
            language = self.language

        return pattern.format(
            tvd=self.tvd_dir,
            series=episode.series,
            season=episode.season,
            episode=episode.episode,
            language=language
        )

    def path_to_subtitles(self, episode, language=None):

        pattern = (
            '{tvd}/{series}/dvd/rip/subtitles/'
            '{series}.Season{season:02d}.Episode{episode:02d}.{language}.srt'
        )

        if language is None:
            language = self.language

        return pattern.format(
            tvd=self.tvd_dir,
            series=episode.series,
            season=episode.season,
            episode=episode.episode,
            language=language
        )

    def path_to_stream(self, episode, format='mp4'):
        """
        Parameters
        ----------
        format : {'mp4', 'ogv', 'webm'}
            Defaults to 'mp4'
        """

        assert format in ['mp4', 'ogv', 'webm']

        pattern = (
            '{tvd}/{series}/dvd/rip/stream/'
            '{series}.Season{season:02d}.Episode{episode:02d}.{format}'
        )

        return pattern.format(
            tvd=self.tvd_dir,
            series=episode.series,
            season=episode.season,
            episode=episode.episode,
            format=format
        )

    def path_to_resource(self, episode, resource):

        pattern = (
            '{tvd}/{series}/metadata/{resource}/'
            '{series}.Season{season:02d}.Episode{episode:02d}.json'
        )

        return pattern.format(
            tvd=self.tvd_dir,
            series=episode.series,
            resource=resource,
            season=episode.season,
            episode=episode.episode
        )
